////////////////////////////////////////////////////////////
//
// SFML - Simple and Fast Multimedia Library
// Copyright (C) 2007 Laurent Gomila (laurent.gom@gmail.com)
//
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//
////////////////////////////////////////////////////////////

#ifndef SFML_EVENT_H
#define SFML_EVENT_H

////////////////////////////////////////////////////////////
// Headers
////////////////////////////////////////////////////////////
#include <SFML/Config.h>


////////////////////////////////////////////////////////////
/// Definition of key codes for keyboard events
////////////////////////////////////////////////////////////
typedef enum
{
    sfKeyA = 'a', sfKeyZ = 'z', sfKeyE = 'e', sfKeyR = 'r', sfKeyT = 't', sfKeyY = 'y', sfKeyU = 'u', sfKeyI = 'i',
    sfKeyO = 'o', sfKeyP = 'p', sfKeyQ = 'q', sfKeyS = 's', sfKeyD = 'd', sfKeyF = 'f', sfKeyG = 'g', sfKeyH = 'h',
    sfKeyJ = 'j', sfKeyK = 'k', sfKeyL = 'l', sfKeyM = 'm', sfKeyW = 'w', sfKeyX = 'x', sfKeyC = 'c', sfKeyV = 'v',
    sfKeyB = 'b', sfKeyN = 'n',
    sfKeyNum0 = '0', sfKeyNum1 = '1', sfKeyNum2 = '2', sfKeyNum3 = '3', sfKeyNum4 = '4',
    sfKeyNum5 = '5', sfKeyNum6 = '6', sfKeyNum7 = '7', sfKeyNum8 = '8', sfKeyNum9 = '9', 
    sfKeyEscape = 256,
    sfKeyControl, sfKeyShift, sfKeyAlt,
    sfKeySpace, sfKeyReturn, sfKeyBack, sfKeyTab, sfKeyPageUp, sfKeyPageDown, sfKeyEnd, sfKeyHome, sfKeyInsert,
    sfKeyDelete, sfKeyAdd, sfKeySubtract, sfKeyMultiply, sfKeyDivide,
    sfKeyLeft, sfKeyRight, sfKeyUp, sfKeyDown,
    sfKeyNumpad0, sfKeyNumpad1, sfKeyNumpad2, sfKeyNumpad3, sfKeyNumpad4, sfKeyNumpad5, sfKeyNumpad6, sfKeyNumpad7,
    sfKeyNumpad8, sfKeyNumpad9,
    sfKeyF1, sfKeyF2, sfKeyF3, sfKeyF4, sfKeyF5, sfKeyF6, sfKeyF7, sfKeyF8, sfKeyF9, sfKeyF10, sfKeyF11, sfKeyF12,
    sfKeyF13, sfKeyF14, sfKeyF15,
    sfKeyPause,

    sfKeyCount // For internal use
} sfKeyCode;


////////////////////////////////////////////////////////////
/// Definition of button codes for mouse events
////////////////////////////////////////////////////////////
typedef enum
{
    sfButtonLeft,
    sfButtonRight,
    sfButtonMiddle,
    sfButtonX1,
    sfButtonX2
} sfMouseButton;


////////////////////////////////////////////////////////////
/// Definition of joystick axis for joystick events
////////////////////////////////////////////////////////////
typedef enum
{
    sfJoyAxisX,
    sfJoyAxisY,
    sfJoyAxisZ,
    sfJoyAxisR,
    sfJoyAxisU,
    sfJoyAxisV,
    sfJoyAxisPOV
} sfJoyAxis;


////////////////////////////////////////////////////////////
/// Definition of all the event types
////////////////////////////////////////////////////////////
typedef enum
{
    sfEvtClosed,
    sfEvtResized,
    sfEvtLostFocus,
    sfEvtGainedFocus,
    sfEvtTextEntered,
    sfEvtKeyPressed,
    sfEvtKeyReleased,
    sfEvtMouseWheelMoved,
    sfEvtMouseButtonPressed,
    sfEvtMouseButtonReleased,
    sfEvtMouseMoved,
    sfEvtJoyButtonPressed,
    sfEvtJoyButtonReleased,
    sfEvtJoyMoved
} sfEventType;


////////////////////////////////////////////////////////////
/// sfEvent defines a system event and its parameters
////////////////////////////////////////////////////////////
typedef union
{
    ////////////////////////////////////////////////////////////
    // Member data
    ////////////////////////////////////////////////////////////
    sfEventType Type; ///< Type of the event

    ////////////////////////////////////////////////////////////
    /// Text event parameters
    ////////////////////////////////////////////////////////////
    struct
    {
        sfEventType Type;
        sfUint16    Unicode;
    } Text;

    ////////////////////////////////////////////////////////////
    /// Keyboard events parameters
    ////////////////////////////////////////////////////////////
    struct
    {
        sfEventType Type;
        sfKeyCode   Code;
        sfBool      Alt;
        sfBool      Control;
        sfBool      Shift;
    } Key;

    ////////////////////////////////////////////////////////////
    /// Mouse move events parameters
    ////////////////////////////////////////////////////////////
    struct
    {
        sfEventType  Type;
        unsigned int X;
        unsigned int Y;
    } MouseMove;

    ////////////////////////////////////////////////////////////
    /// Mouse button events parameters
    ////////////////////////////////////////////////////////////
    struct
    {
        sfEventType   Type;
        sfMouseButton Button;
    } MouseButton;

    ////////////////////////////////////////////////////////////
    /// Mouse wheel events parameters
    ////////////////////////////////////////////////////////////
    struct
    {
        sfEventType Type;
        int         Delta;
    } MouseWheel;

    ////////////////////////////////////////////////////////////
    /// Joystick move events parameters
    ////////////////////////////////////////////////////////////
    struct
    {
        sfEventType  Type;
        unsigned int JoystickId;
        sfJoyAxis    Axis;
        float        Position;
    } JoyMove;

    ////////////////////////////////////////////////////////////
    /// Joystick button events parameters
    ////////////////////////////////////////////////////////////
    struct
    {
        sfEventType  Type;
        unsigned int JoystickId;
        unsigned int Button;
    } JoyButton;

    ////////////////////////////////////////////////////////////
    /// Size events parameters
    ////////////////////////////////////////////////////////////
    struct
    {
        sfEventType  Type;
        unsigned int Width;
        unsigned int Height;
    } Size;
} sfEvent;


#endif // SFML_EVENT_H
